/*

Copyright 2008 TOPdesk, the Netherlands

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/

package org.mangosdk.spi.processor.testutils;

import java.util.Map;

import org.mangosdk.spi.processor.Initializer;

public final class TestInitializer implements Initializer {

	private final Map<String, String> data;

	public TestInitializer(Map<String, String> data) {
		this.data = data;
	}
	
	@Override
	public CharSequence initialData(String serviceName) {
		return data.get(serviceName);
	}
}
